package com.caimei.mapper;

import com.caimei.model.po.*;
import com.caimei.model.vo.CartProductVo;
import com.caimei.model.vo.ClubVo;
import com.caimei.model.vo.LadderPriceVo;
import com.caimei.model.vo.ProductVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/3/23
 */
@Mapper
public interface ProductMapper {
    /**
     * 查询组织机构
     *
     * @param organizeId 组织id
     * @param name
     * @return
     */
    List<ClubVo> findClubList(@Param("organizeId") Integer organizeId, @Param("name") String name);

    /**
     * 查询组织商品
     *
     * @param organizeId
     * @param productName
     * @return
     */
    List<ProductVo> findProductList(@Param("organizeId") Integer organizeId, @Param("productName") String productName);

    /**
     * 根据商品Id查询促销活动
     *
     * @param productId
     * @return
     */
    PromotionsPo findPromotionByProductId(Integer productId);

    /**
     * 获取阶梯价格
     *
     * @param productId
     * @return
     */
    List<LadderPriceVo> findLadderPriceByProductId(Integer productId);

    /**
     * 查询商品
     *
     * @param productId
     * @return
     */
    ProductVo findProductByProductId(Integer productId);

    /**
     * 查询商品图片
     *
     * @param originalProductId 原采美商品Id
     * @return
     */
    List<ProductImagePo> findProductImagesByOriginalId(Integer originalProductId);

    /**
     * 商品详细信息
     *
     * @param originalProductId
     * @return
     */
    ProductDetailInfoPo findProductDetailByOriginalId(Integer originalProductId);

    /**
     * 商品分类名称
     *
     * @param bigTypeId
     * @param smallTypeId
     * @param tinyTypeId
     * @return
     */
    String findTypeName(@Param("bigTypeId") Integer bigTypeId, @Param("smallTypeId") Integer smallTypeId, @Param("tinyTypeId") Integer tinyTypeId);

    /**
     * @param brandId
     * @return
     */
    String findBrandNameByBrandId(Integer brandId);

    List<RelatedParametersPo> findParametersByOriginalId(Integer originalProductId);

    /**
     * 查询组织商品信息
     *
     * @param productId 组织商品id
     * @return
     */
    CmOrganizeProductPo getProduct(Integer productId);

    /**
     * 查询促销活动信息
     *
     * @param promotionsId
     * @return
     */
    PromotionsPo findParameterById(Integer promotionsId);

    /**
     * 查询采美运费商品
     *
     * @return
     */
    ProductPo findPostFeeProduct();
}
