package com.caimei.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.caimei.mapper.LoginMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.dto.BuyerUserDto;
import com.caimei.model.po.CmMallOrganizePo;
import com.caimei.model.po.CmOrganizeBuyerPo;
import com.caimei.model.vo.BuyerUserVo;
import com.caimei.service.LoginService;
import com.caimei.util.HttpRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/3/22
 */
@Slf4j
@Service
public class LoginServiceImpl implements LoginService {
    @Resource
    private LoginMapper loginMapper;

    @Value("${wx.AppId}")
    private String appId;

    @Value("${wx.AppSecret}")
    private String appSecret;

    @Override
    public ResponseJson<BuyerUserVo> authorizationLogin(String code) {
        String requestUrl = "https://api.weixin.qq.com/sns/jscode2session";
        Map<String, String> requestUrlParam = new HashMap<>(4);
        //小程序appId
        requestUrlParam.put("appid", appId);
        //小程序appSecret
        requestUrlParam.put("secret", appSecret);
        //小程序端返回的code
        requestUrlParam.put("js_code", code);
        //默认参数
        requestUrlParam.put("grant_type", "authorization_code");
        //发送post请求读取调用微信接口获取openid用户唯一标识
        String infos;
        try {
            infos = HttpRequest.sendPost(requestUrl, requestUrlParam);
        } catch (Exception e) {
            log.info("微信服务器异常", e);
            return ResponseJson.error("微信服务器异常", null);
        }
        //解析相应内容（转换成json对象）
        JSONObject jsonObject = JSON.parseObject(infos);
        String openid = jsonObject.getString("openid");
        log.info("小程序openid----->" + openid);
        String errCode = jsonObject.getString("errcode");
        String errMsg = jsonObject.getString("errmsg");
        if (!StringUtils.isEmpty(errCode) && ("-1".equals(errCode) || "40029".equals(errCode) || "45011".equals(errCode))) {
            log.error(errMsg);
            return ResponseJson.error("授权登录失败", null);
        }
        BuyerUserVo buyerUser = loginMapper.findBuyer(openid);
        if (buyerUser == null) {
            buyerUser = new BuyerUserVo();
            buyerUser.setOpenid(openid);
            return ResponseJson.error("未绑定微信", buyerUser);
        }
        return ResponseJson.success(buyerUser);
    }

    @Override
    public ResponseJson<BuyerUserVo> invitationCode(BuyerUserDto buyerUserDto) {
        CmOrganizeBuyerPo organizeBuyer = loginMapper.findByInvitationCode(buyerUserDto.getInvitationCode());
        if (organizeBuyer == null) {
            return ResponseJson.error("邀请码错误，请联系采美客服获取最新邀请码", null);
        }
        if ("2".equals(organizeBuyer.getStatus())) {
            return ResponseJson.error("邀请码已被使用，请联系采美客服获取最新邀请码", null);
        }
        if (organizeBuyer.getInvitationCodeTime().compareTo(new Date()) < 0) {
            return ResponseJson.error("邀请码已失效，请联系采美客服获取最新邀请码", null);
        }
        CmMallOrganizePo organize = loginMapper.findOrganize(organizeBuyer.getOrganizeId());
        if (organize == null) {
            return ResponseJson.error("您所属公司已下线，暂不能登录。请联系采美客服了解详情", null);
        }
        loginMapper.updateBuyer(buyerUserDto);
        BuyerUserVo buyerUser = new BuyerUserVo();
        buyerUser.setOrganizeId(organize.getId());
        buyerUser.setOpenid(buyerUser.getOpenid());
        return ResponseJson.success(buyerUser);
    }

    @Override
    public ResponseJson<Map<String, Object>> personalData(Integer organizeId) {
        Map<String, Object> map = new HashMap<>(5);
        CmMallOrganizePo organize = loginMapper.findOrganize(organizeId);
        map.put("organizeName", organize.getOrganizeName());
        //待确认数量
        Integer confirmedCount = loginMapper.findOrderCount(organizeId, 0);
        map.put("confirmedCount", confirmedCount);
        //待付款数量
        Integer paymentCount = loginMapper.findOrderCount(organizeId, 1);
        paymentCount = paymentCount == null ? 0 : paymentCount;
        map.put("paymentCount", paymentCount);
        //待发货数量
        Integer waitShipmentsCount = loginMapper.findOrderCount(organizeId, 2);
        waitShipmentsCount = waitShipmentsCount == null ? 0 : waitShipmentsCount;
        map.put("waitShipmentsCount", waitShipmentsCount);
        //已发货数量
        Integer shipmentsCount = loginMapper.findOrderCount(organizeId, 3);
        shipmentsCount = shipmentsCount == null ? 0 : shipmentsCount;
        map.put("shipmentsCount", shipmentsCount);
        //退货款数量
        Integer salesReturnCount = loginMapper.findOrderCount(organizeId, 4);
        salesReturnCount = salesReturnCount == null ? 0 : salesReturnCount;
        map.put("salesReturnCount", salesReturnCount);
        return ResponseJson.success(map);
    }
}
